<?php
namespace Teknisa\EST\Modules\Relatorios\Services;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;

class Est70000ResumoDeOperacoesService {

    protected $factory;
    protected $environment;
    protected $CDSUBGRPROD;
    protected $daysBetweenDate;

	public function __construct($factory, $environment, NewCode $newCode) {
        $this->environment = $environment;
	    $this->newCode = $newCode;
		$this->factory = $factory;
	}
	
	// This function gets groupFili to verifies if it owns main group or groups by unity
	public function getGroupFiliInfo ($CDGRUPPROD){
		$params = ['CDGRUPPROD' => $CDGRUPPROD ];
		return $this->factory->getGroupFiliInfo($params);
	}
	
    // This function Manages final dataSet
	public function dataSetBuilder($columns, $hasMainGroup, $params){

        $params['CDFILISUBGR'] = $hasMainGroup ? "P" : $params['CDFILIAL'] ;	 
	    
	    // get info according to the needs 
	    $sales = $this->factory->getSalesInfo($params);
	    $nf = $this->factory->getNfInfo($params);
	    $transfer = $this->factory->getTransferInfo($params);
	    $averageStock = $this->getAverageStockManager($params);
	    $checkoutConter = $this->getCheckoutCounterInfo($params);
	    
	    $nrSequential = $this->newCode->getCodeByTableName('MOVIESTOQUE', $this->environment->getNrOrgTrab());
		$procedureParams = [
    	    "P_FILIAL"=> $params['CDFILIAL'],
            "P_CDALMOXARIFE"=> $params['CDALMOXARIFE'] != ' ' ? $params['CDALMOXARIFE'] : null,
            "P_CDLOCALESTOQ"=> $params['CDLOCALESTOQ'] != ' ' ? $params['CDLOCALESTOQ'] : null,
            "P_NRLOTEESTQ"=> null,
            "P_TAMNIVEL"=> 10,
            "P_DTINICIAL"=> $params['DTINICIAL'],
            "P_DTFINAL"=> $params['DTFINAL'],
            "P_PRODINI"=> $this->factory->getFirstCdArvprod(),
            "P_PRODFIM"=> 'ZZZZZZZZZZZZZZZZZZZZ',
            "P_IDLANCTO"=>3,
            "P_IDCUSTO"=> 1,
            "P_SEQUMOVI"=>$nrSequential
		];
		$this->factory->executeProcedureMovimentacaoEstoque($procedureParams);
        
	    $stockServices = $this->getStockServicesManager($params,$nrSequential);
        
        // It builds collumns with specifc order.
        $rowsMetadata = $this->rowMetadataBuilder();
        
        foreach($rowsMetadata as $field => $title){
            switch($field){
                case "VLRLIQUIDO":
                case "VLRBRUTO":
                case "VRDEVOLUCAO":
                    $dataSet[] = $this->rowBuilder($columns,$sales,$field);
                    break;
                case "VLRTOTALNF":
                    $dataSet[] = $this->rowBuilder($columns,$nf,$field);
                    break;
                case "VLRTSIMPENTR":
                case "VLRTSIMPSAID":
                    $dataSet[] = $this->rowBuilder($columns,$transfer,$field);
                    break;
                case "VRESTOQDIAF":
                    $dataSet[] = $this->rowBuilder($columns,$averageStock,$field);
                    break;
                case "VRCONSEMPRESA":
                case "VRCONSFUNC":
                case "VALRPOSINI":
                case "VALRPOSFIM":
                case "VRFALTAINV":
                case "VRSOBRAINV":
                    $dataSet[] = $this->rowBuilder($columns,$stockServices,$field);
                    break;
                case "VRTICKETCAIXA":
                case "QTTICKETCAIXA":
                case "QTDEITENS":
                case "VRTICKETMEDIO":
                case "QTITPTC":
                    $dataSet[] = $this->totalBuilder($columns,$checkoutConter,$field);
                    break;
                case "VRCUSTOTOT":
                case "VRCOMPPROD":
                case "VRDIVER":
                case "VRBASECALC":
                case "QTDIASEST":
                    $dataSet[] = $this->calcFieldsBuilder($columns,$dataSet,$sales,$nf,$transfer,$stockServices,$field,$params);
                    break;
                default:
                    break;
            }
        }
        
        $this->onChangeTitles($dataSet);
        
        return $dataSet;
    }
    
    //This create the rows of the dataSet and the order of it.
    public function rowMetadataBuilder(){
	    // !!!WARNING!!!
	    // Do not change the order keys, it will change the order of the rows in the final grid.
        
        return [
            "VLRLIQUIDO" => "Venda Líquida",
            "VLRBRUTO" => "Venda Bruta",
            "VLRTOTALNF" => "Entradas de NF's",
            "VRDEVOLUCAO" => "Devolução de Compra",
            "VLRTSIMPENTR" => "Transf. Simplificada Entrada",
            "VLRTSIMPSAID" => "Transf. Simplificada Saída",
            "VRCONSEMPRESA" => "Consumo Empresa (Saída)", 
            "VRCONSFUNC" => "Consumo Funcionário (Saída)", 
            "VALRPOSINI" => "Estoque Inicial", 
            "VALRPOSFIM" => "Estoque Final", 
            "VRCUSTOTOT" => "Custo Total", 
            "VRCOMPPROD" => "Composição do Produto", 
            "VRFALTAINV" => "Falta de Inventário", 
            "VRSOBRAINV" => "Sobra de Inventário",
            "VRDIVER" => "Divergencia", 
            "VRCONSPERDA" => "Perdas", 
            "VRESTOQDIAF" => "Estoque Médio", 
            "VRBASECALC" => "Base de Cálculo", 
            "QTDIASEST" => "Dias de Estoque",
            "VRTICKETCAIXA" => "Ticket's Caixa",
            "QTTICKETCAIXA" => "Quantidade Ticket's Caixa",
            "QTDEITENS" => "Quantidade Itens",
            "VRTICKETMEDIO" => "Ticket's Médio",
            "QTITPTC" => "Itens P/T.C"
        ];
    }
    
    //This function builds a each row
    public function rowBuilder($columns,$dataSet,$field){
        $total = 0;
        foreach($dataSet as $data){
            foreach($data as $key => $value){
                foreach($columns as $column){
                    if($key === $field && $data['CDSUBGRPROD'] == $column['CDSUBGRPROD']){
                        if(!isset($row["TITLE"]))
                            $row[] = ["TITLE" => $field];
                        $row[] = [$column['CDSUBGRPROD'] => $data[$key]]; 
                        $total += $data[$key] ?? 0;
                    }
                }
            }
        }
        
        $row[] = ['TOTAL' => $total];
        // This function works like flat function in Js 
        return call_user_func_array('array_merge', $row);
        
    }
    
    //This function builds a each total row
    public function totalBuilder($columns,$dataSet,$field){
        $row = [];
        if(!isset($row["TITLE"]))
            $row[] = ["TITLE" => $field];
        for($i = 0; $i < count($columns); $i++){
            if($i == 1){
                if(!empty($dataSet))
                    $row[] = [$columns[$i]['CDSUBGRPROD'] => $dataSet[0][$field]]; 
                else
                    $row[] = [$columns[$i]['CDSUBGRPROD'] => 0]; 
            }else
                $row[] = [$columns[$i]['CDSUBGRPROD'] => 'n/a']; 
            
        }
        $row[] = ['TOTAL' => 'n/a'];
        // This function works like flat function in Js 
        return call_user_func_array('array_merge', $row);
    }
    
    //This get average stock dataSet
    public function getAverageStockManager($params){
        
        $dtIni = DateUtil::getDataDeString($params['DTINICIAL']);
        $dtFin = DateUtil::getDataDeString($params['DTFINAL']);
        $dtIniPlusOne = date_add(DateUtil::getDataDeString($params['DTINICIAL']), date_interval_create_from_date_string("1 days"));
        $this->daysBetweenDate = DateUtil::getQtdeDiasEntre($dtIniPlusOne->format('d/m/Y'),$params['DTFINAL']);
        $dataSet = [];
        
        unset($params['DTINICIAL']);
        unset($params['DTFINAL']);
        if($dtIni < $dtFin){
            while($dtIni < $dtFin){
                $params['DTPOSIESTQ'] = $dtIni->format('d/m/Y');
                $results = $this->factory->getAverageStockInfo($params);
                
                if(!empty($dataSet)){
                    foreach($results as $result){
                        foreach($dataSet as &$group){
                            if($group["CDSUBGRPROD"] == $result["CDSUBGRPROD"]){
                                $group["VRESTOQDIAF"] += $result["VRESTOQDIAF"];
                                
                                if(DateUtil::getQtdeDiasEntre($dtIni,$dtFin) == 1){
                                    $daysBetweenDate = $this->daysBetweenDate >= 0 ? $this->daysBetweenDate : 1;
                                    $group["VRESTOQDIAF"] = $group["VRESTOQDIAF"] / $daysBetweenDate;
                                }
                            }
                        }
                    }
                }else
                    $dataSet = $results;
                $dtIni->modify('+1 day');
            }
        }else{
            $dtIni->modify('+1 day');
            $params['DTPOSIESTQ'] = $dtIni->format('d/m/Y');
            $dataSet = $this->factory->getAverageStockInfo($params);
        }
        return $dataSet;
    }
    
    //This get stock services dataSet
    public function getStockServicesManager($params,$nrSequential){
        $params['NRRELATMOVI'] = $nrSequential;
        $params['DTPOSIESTQ'] = $params['DTFINAL'];
        unset($params['DTINICIAL']);
        unset($params['DTFINAL']);
        
        return $this->factory->getStockServicesInfo($params);
    }
    
    //This get checkoutConter dataSet
    public function getCheckoutCounterInfo($params){
        unset($params['CDGRUPPROD']);
        unset($params['CDFILISUBGR']);
        
        return $this->factory->getCheckoutCounterInfo($params);
    }
    
    //This function gets values of calculated fileds
    public function calcFieldsBuilder($columns,$originalDataSet, $salesData,$nfData,$transferData,$stockServicesData,$field,$params){
        $row = [];
        $total = 0;
        foreach($columns as $column){
            $this->CDSUBGRPROD = $column['CDSUBGRPROD'];
            
            $sales = $this->getSubGrupInfoFromDataSet($salesData);
            $nf = $this->getSubGrupInfoFromDataSet($nfData);
            $transfer = $this->getSubGrupInfoFromDataSet($transferData);
            $stockServices = $this->getSubGrupInfoFromDataSet($stockServicesData);
            
            switch($field){
                case "VRCUSTOTOT":
                case "VRCOMPPROD":
                case "VRBASECALC":
                    $value = (
                        $nf['VLRTOTALNF'] + 
                        $transfer['VLRTSIMPENTR'] + 
                        $stockServices['VALRPOSINI']
                    ) - (
                        $sales['VRDEVOLUCAO'] + 
                        $transfer['VLRTSIMPSAID'] + 
                        $stockServices['VRCONSEMPRESA'] + 
                        $stockServices['VALRPOSFIM'] + 
                        $stockServices['VRCONSFUNC']
                    );
                    break;
                case "VRDIVER":
                    $value = $stockServices['VRFALTAINV'] - $stockServices['VRSOBRAINV'];
                    break;
                case "QTDIASEST":
                    $originalColumn = array_filter($originalDataSet, function($field){
                        return $field['TITLE'] == 'VRESTOQDIAF';
                    });
                    
                    $ticketMedio = call_user_func_array('array_merge', $originalColumn);
                    $ticketMedio = array_filter($ticketMedio, function($field){
                        return $field == $this->CDSUBGRPROD;
                    },ARRAY_FILTER_USE_KEY);
                    
                    $valueCheck = (
                        $nf['VLRTOTALNF'] + 
                        $transfer['VLRTSIMPENTR'] + 
                        $stockServices['VALRPOSINI']
                    ) - (
                        $sales['VRDEVOLUCAO'] + 
                        $transfer['VLRTSIMPSAID'] + 
                        $stockServices['VRCONSEMPRESA'] + 
                        $stockServices['VALRPOSFIM'] + 
                        $stockServices['VRCONSFUNC']
                    );
                    if($valueCheck != 0){
                        $daysBetweenDate = $this->daysBetweenDate >= 0 ? $this->daysBetweenDate : 1;
                        $vrTicketMedio = $ticketMedio[$this->CDSUBGRPROD] != 0 ? $ticketMedio[$this->CDSUBGRPROD] : 1;
                        $valueCheck = $valueCheck != 0 ? $valueCheck : 1;
                
                        $value = $valueCheck / ($vrTicketMedio / $daysBetweenDate);
                    }
                    else
                        $value = 0;
                    
                    break;
            }
            
            if(!isset($row["TITLE"]))
                $row[] = ["TITLE" => $field];
            $row[] = [$column['CDSUBGRPROD'] => $value];
            if($field == 'QTDIASEST')
                $total = 'n/a';
            else
                $total += $value ?? 0;
        }
        $row[] = ['TOTAL' => $total];
        // This function works like flat function in Js 
        return call_user_func_array('array_merge', $row);
    }
    
    //This function get data to a specifc subGroup.
    public function getSubGrupInfoFromDataSet($dataSet){
        $arr = array_filter($dataSet, function($field){
            return $field['CDSUBGRPROD'] == $this->CDSUBGRPROD;
        });
        return call_user_func_array('array_merge', $arr);
    }
    
    //It changes the title field values
    public function onChangeTitles(&$dataSet){
        $rowsMetadata = $this->rowMetadataBuilder();
        
        foreach($dataSet as &$row){
            foreach($rowsMetadata as $key => $value){
                if($row['TITLE'] == $key){
                    $row['TITLE'] = $value;
                }
            }
        }
    }
}

