<?php

namespace Teknisa\EST\Services;

use Teknisa\Libs\Util\DateUtil;

class FecharDia {
    protected $entityManager;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $dataSourceManager;
    protected $fecharDiaFactory;

    public function __construct($entityManager, $crudService, $parameterBag, $dataSourceManager, $fecharDiaFactory) {
        $this->entityManager     = $entityManager;
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->fecharDiaFactory  = $fecharDiaFactory;
    }

    public function verificaInventProg($CDFILIAL, $DIAFECHAMES, $NRORG) {
        $sql = "SELECT NRDIAINVFILI
                  FROM DIAINVEFILI
                 WHERE CDFILIAL     = :P_CDFILIAL
                   AND NRDIAINVFILI = :P_DIAFECHAMES
                   AND NRORG        = :P_NRORG";

        $params = array("P_CDFILIAL"    => $CDFILIAL,
                        "P_DIAFECHAMES" => $DIAFECHAMES,
                        "P_NRORG"       => $NRORG);

        $connection = $this->entityManager->getConnection();

        $query = $connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function deleteInveRota($CDFILIAL, $NRORG) {
        $sql = "DELETE
                  FROM INVEROTALOTE
                 WHERE CDALMOXARIFE IN (SELECT CDALMOXARIFE
                                          FROM ALMOXARIFADO
                                         WHERE CDFILIAL      = :CDFILIAL
                                           AND CDALMOXARIFE <> ' '
                                           AND NRORG         = :NRORG)";

        $params = array("CDFILIAL" => $CDFILIAL,
                        "NRORG"    => $NRORG);

        $connection = $this->entityManager->getConnection();

        $query = $connection->prepare($sql);
        $query->execute($params);
    }

    public function saveItemInverota($CDFILIAL, $DTINVEROTA, $CDPRODUTO, $QTESTOQDIA, $CDALMOXARIFE, $CDLOCALESTOQ, $QTINVEROTA, $NRORG) {
        $sql = "INSERT INTO INVEROTA (CDFILIAL,
                                      DTINVEROTA,
                                      CDPRODUTO,
                                      QTINVEROTA,
                                      QTESTOQUE,
                                      CDALMOXARIFE,
                                      CDLOCALESTOQ,
                                      NRORG)
                             (SELECT :CDFILIAL,
                                     :DTINVEROTA,
                                     :CDPRODUTO,
                                     :QTINVEROTA,
                                     :QTESTOQUE,
                                     :CDALMOXARIFE,
                                     :CDLOCALESTOQ,
                                     :NRORG
                                 FROM DUAL
                                WHERE NOT EXISTS
                              (SELECT CDFILIAL,
                                      DTINVEROTA,
                                      CDPRODUTO,
                                      QTINVEROTA,
                                      QTESTOQUE,
                                      CDALMOXARIFE,
                                      CDLOCALESTOQ,
                                      NRORG
                                 FROM INVEROTA
                                WHERE CDFILIAL     = :CDFILIAL
                                  AND CDPRODUTO    = :CDPRODUTO
                                  AND CDALMOXARIFE = :CDALMOXARIFE
                                  AND CDLOCALESTOQ = :CDLOCALESTOQ
                                  AND DTINVEROTA   = :DTINVEROTA
                                  AND NRORG        = :NRORG))";

        $params = array("CDFILIAL"     => $CDFILIAL,
                        "DTINVEROTA"   => $DTINVEROTA,
                        "CDPRODUTO"    => $CDPRODUTO,
                        "QTESTOQUE"    => $QTESTOQDIA,
                        "CDALMOXARIFE" => $CDALMOXARIFE,
                        "CDLOCALESTOQ" => $CDLOCALESTOQ,
                        "QTINVEROTA"   => $QTINVEROTA,
                        "NRORG"        => $NRORG);

        $connection = $this->entityManager->getConnection();

        $query = $connection->prepare($sql);
        $query->execute($params);
    }

    public function produtosInventProg($CDFILIAL, $DTPOSICAO, $CDALMOXARIFE, $CDLOCALESTOQ, $NRORG) {
      $sql = "SELECT DISTINCT D.CDPRODUTO,
                              P.CDPRODESTO,
                              D.QTESTOQDIA,
                              D.NRSUBLOTE,
                              D.NRLOTEESTQ
                         FROM PRODUTO P,
                              POSESTDIA D,
                              PRODFILI F,
                              PRODFILI C,
                      (SELECT CDFILIAL,
                              CDALMOXARIFE,
                              CDLOCALESTOQ,
                              CDPRODUTO,
                              NRLOTEESTQ,
                              NRSUBLOTE,
                              MAX(DTPOSIESTQ) AS DTPOSIESTQ
                         FROM POSESTDIA
                        WHERE DTPOSIESTQ <= :P_DTPOSICAO
                          AND NRORG       = :P_NRORG
                     GROUP BY CDFILIAL,
                              CDALMOXARIFE,
                              CDLOCALESTOQ,
                              CDPRODUTO,
                              NRLOTEESTQ,
                              NRSUBLOTE) AX
                        WHERE D.CDFILIAL     = :P_CDFILIAL
                          AND D.CDALMOXARIFE = :P_CDALMOXARIFE
                          AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
                          AND D.CDPRODUTO    =  P.CDPRODUTO
                          AND P.CDPRODESTO   =  F.CDPRODUTO
                          AND F.CDFILIAL     =  D.CDFILIAL
                          AND D.CDFILIAL     = AX.CDFILIAL
                          AND D.CDALMOXARIFE = AX.CDALMOXARIFE
                          AND D.CDLOCALESTOQ = AX.CDLOCALESTOQ
                          AND D.CDPRODUTO    = AX.CDPRODUTO
                          AND D.NRLOTEESTQ   = AX.NRLOTEESTQ
                          AND D.NRSUBLOTE    = AX.NRSUBLOTE
                          AND D.DTPOSIESTQ   = AX.DTPOSIESTQ
                          AND F.IDCNTRESTOQ  = 'S'
                          AND F.IDPRODATIV   = 'A'
                          AND P.IDPRODATIVPR = 'S'
                          AND C.CDFILIAL     =  D.CDFILIAL
                          AND C.CDPRODUTO    =  D.CDPRODUTO
                          AND C.IDPRODATIV   = 'A'
                          AND P.NRORG        = :P_NRORG
                          AND P.NRORG        =  D.NRORG
                          AND P.NRORG        =  F.NRORG
                          AND P.NRORG        =  C.NRORG";

      $params = array("P_CDFILIAL"     => $CDFILIAL,
                      "P_DTPOSICAO"    => $DTPOSICAO,
                      "P_CDALMOXARIFE" => $CDALMOXARIFE,
                      "P_CDLOCALESTOQ" => $CDLOCALESTOQ,
                      "P_NRORG"        => $NRORG);

      $connection = $this->entityManager->getConnection();

      $query = $connection->prepare($sql);
      $query->execute($params);

      return $query->fetchAll();
    }

    public function produtosInventProgAlmoxarife($CDFILIAL, $DTPOSICAO, $CDLOCALESTOQ, $NRORG) {
        $sql = "SELECT DISTINCT D.CDPRODUTO,
                                P.CDPRODESTO,
                                D.QTESTOQDIA,
                                D.CDALMOXARIFE,
                                D.NRSUBLOTE,
                                D.NRLOTEESTQ
                           FROM PRODUTO P,
                                POSESTDIA D,
                                PRODFILI F,
                                PRODFILI C,
                        (SELECT CDFILIAL,
                                CDALMOXARIFE,
                                CDLOCALESTOQ,
                                CDPRODUTO,
                                NRLOTEESTQ,
                                NRSUBLOTE,
                                MAX(DTPOSIESTQ) AS DTPOSIESTQ
                           FROM POSESTDIA
                          WHERE DTPOSIESTQ <= :P_DTPOSICAO
                            AND NRORG       = :P_NRORG
                       GROUP BY CDFILIAL,
                                CDALMOXARIFE,
                                CDLOCALESTOQ,
                                CDPRODUTO,
                                NRLOTEESTQ,
                                NRSUBLOTE) AX
                          WHERE D.CDFILIAL      = :P_CDFILIAL
                            AND D.CDLOCALESTOQ  = :P_CDLOCALESTOQ
                            AND D.CDALMOXARIFE <> ' '
                            AND D.CDPRODUTO     =  P.CDPRODUTO
                            AND P.CDPRODESTO    =  F.CDPRODUTO
                            AND F.CDFILIAL      =  D.CDFILIAL
                            AND D.CDFILIAL      = AX.CDFILIAL
                            AND D.CDALMOXARIFE  = AX.CDALMOXARIFE
                            AND D.CDLOCALESTOQ  = AX.CDLOCALESTOQ
                            AND D.CDPRODUTO     = AX.CDPRODUTO
                            AND D.NRLOTEESTQ    = AX.NRLOTEESTQ
                            AND D.NRSUBLOTE     = AX.NRSUBLOTE
                            AND D.DTPOSIESTQ    = AX.DTPOSIESTQ
                            AND F.IDCNTRESTOQ   = 'S'
                            AND F.IDPRODATIV    = 'A'
                            AND P.IDPRODATIVPR  = 'S'
                            AND C.CDFILIAL      =  D.CDFILIAL
                            AND C.CDPRODUTO     =  D.CDPRODUTO
                            AND C.IDPRODATIV    = 'A'
                            AND P.NRORG         = :P_NRORG
                            AND P.NRORG         =  D.NRORG
                            AND P.NRORG         =  F.NRORG
                            AND P.NRORG         =  C.NRORG";

        $params = array("P_CDFILIAL"     => $CDFILIAL,
                        "P_DTPOSICAO"    => $DTPOSICAO,
                        "P_CDLOCALESTOQ" => $CDLOCALESTOQ,
                        "P_NRORG"        => $NRORG);

        $connection = $this->entityManager->getConnection();

        $query = $connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function produtosInventProgAlmoLocal($CDFILIAL, $DTPOSICAO, $NRORG) {
      $sql = "SELECT DISTINCT D.CDPRODUTO,
                              P.CDPRODESTO,
                              D.QTESTOQDIA,
                              D.CDALMOXARIFE,
                              D.CDLOCALESTOQ,
                              D.NRSUBLOTE,
                              D.NRLOTEESTQ
                         FROM PRODUTO P,
                              POSESTDIA D,
                              PRODFILI F,
                              PRODFILI C,
                      (SELECT CDFILIAL,
                              CDALMOXARIFE,
                              CDLOCALESTOQ,
                              CDPRODUTO,
                              NRLOTEESTQ,
                              NRSUBLOTE,
                              MAX(DTPOSIESTQ) AS DTPOSIESTQ
                         FROM POSESTDIA
                        WHERE DTPOSIESTQ <= :P_DTPOSICAO
                          AND NRORG       = :P_NRORG
                     GROUP BY CDFILIAL,
                              CDALMOXARIFE,
                              CDLOCALESTOQ,
                              CDPRODUTO,
                              NRLOTEESTQ,
                              NRSUBLOTE) AX
                        WHERE D.CDFILIAL      = :P_CDFILIAL
                          AND D.CDALMOXARIFE <> ' '
                          AND D.CDPRODUTO     =  P.CDPRODUTO
                          AND P.CDPRODESTO    =  F.CDPRODUTO
                          AND F.CDFILIAL      =  D.CDFILIAL
                          AND D.CDFILIAL      = AX.CDFILIAL
                          AND D.CDALMOXARIFE  = AX.CDALMOXARIFE
                          AND D.CDLOCALESTOQ  = AX.CDLOCALESTOQ
                          AND D.CDPRODUTO     = AX.CDPRODUTO
                          AND D.NRLOTEESTQ    = AX.NRLOTEESTQ
                          AND D.NRSUBLOTE     = AX.NRSUBLOTE
                          AND D.DTPOSIESTQ    = AX.DTPOSIESTQ
                          AND F.IDCNTRESTOQ   = 'S'
                          AND F.IDPRODATIV    = 'A'
                          AND P.IDPRODATIVPR  = 'S'
                          AND C.CDFILIAL      =  D.CDFILIAL
                          AND C.CDPRODUTO     =  D.CDPRODUTO
                          AND C.IDPRODATIV    = 'A'
                          AND P.NRORG         = :P_NRORG
                          AND P.NRORG         =  D.NRORG
                          AND P.NRORG         =  F.NRORG
                          AND P.NRORG         =  C.NRORG";

      $params = array("P_CDFILIAL"  => $CDFILIAL,
                      "P_DTPOSICAO" => $DTPOSICAO,
                      "P_NRORG"     => $NRORG);

      $connection = $this->entityManager->getConnection();

      $query = $connection->prepare($sql);
      $query->execute($params);

      return $query->fetchAll();
    }

    public function deleteInveRotaLote($CDFILIAL, $NRORG) {
      $sql = "DELETE
                FROM INVEROTALOTE
               WHERE CDALMOXARIFE IN
             (SELECT CDALMOXARIFE
                FROM ALMOXARIFADO
               WHERE CDFILIAL      = :CDFILIAL
                 AND CDALMOXARIFE <> ' '
                 AND NRORG         = :NRORG)";

      $params = array("CDFILIAL" => $CDFILIAL,
                      "NRORG"    => $NRORG);

      $connection = $this->entityManager->getConnection();

      $query = $connection->prepare($sql);
      $query->execute($params);
    }

    public function saveItemInverotaLote($CDFILIAL, $DTINVELOTE, $CDPRODUTO, $CDALMOXARIFE, $CDLOCALESTOQ, $NRORG, $NRLOTEESTQ, $NRSUBLOTE, $QTESTOLOTE, $QTINVELOTE) {
      $sql = "INSERT INTO INVEROTALOTE (DTINVELOTE,
                                        CDALMOXARIFE,
                                        CDLOCALESTOQ,
                                        NRLOTEESTQ,
                                        QTESTOLOTE,
                                        CDPRODUTO,
                                        QTINVELOTE,
                                        NRORG,
                                        CDFILIAL,
                                        NRSUBLOTE)
                               (SELECT :DTINVELOTE,
                                       :CDALMOXARIFE,
                                       :CDLOCALESTOQ,
                                       :NRLOTEESTQ,
                                       :QTESTOLOTE,
                                       :CDPRODUTO,
                                       :QTINVELOTE,
                                       :NRORG,
                                       :CDFILIAL,
                                       :NRSUBLOTE
                                   FROM DUAL
                                  WHERE NOT EXISTS
                                (SELECT DTINVELOTE,
                                        CDALMOXARIFE,
                                        CDLOCALESTOQ,
                                        NRLOTEESTQ,
                                        QTESTOLOTE,
                                        CDPRODUTO,
                                        QTINVELOTE,
                                        NRORG,
                                        CDFILIAL,
                                        NRSUBLOTE
                                   FROM INVEROTALOTE
                                  WHERE CDFILIAL     = :CDFILIAL
                                    AND CDPRODUTO    = :CDPRODUTO
                                    AND CDALMOXARIFE = :CDALMOXARIFE
                                    AND CDLOCALESTOQ = :CDLOCALESTOQ
                                    AND DTINVELOTE   = :DTINVELOTE
                                    AND NRLOTEESTQ   = :NRLOTEESTQ
                                    AND QTESTOLOTE   = :QTESTOLOTE
                                    AND QTINVELOTE   = :QTINVELOTE
                                    AND NRSUBLOTE    = :NRSUBLOTE
                                    AND NRORG        = :NRORG))";

      $params = array("CDFILIAL"     => $CDFILIAL,
                      "CDPRODUTO"    => $CDPRODUTO,
                      "CDALMOXARIFE" => $CDALMOXARIFE,
                      "CDLOCALESTOQ" => $CDLOCALESTOQ,
                      "DTINVELOTE"   => $DTINVELOTE,
                      "NRLOTEESTQ"   => $NRLOTEESTQ,
                      "QTESTOLOTE"   => $QTESTOLOTE,
                      "QTINVELOTE"   => $QTINVELOTE,
                      "NRSUBLOTE"    => $NRSUBLOTE,
                      "NRORG"        => $NRORG);

      $connection = $this->entityManager->getConnection();

      $query = $connection->prepare($sql);
      $query->execute($params);
    }

    public function fechaMesTi($DTPROCESSA, $primeiroDiaDTPROCESSA, $ultimoDiaDTPROCESSA, $proxDTPROCESSA, $dia, $NRDIAFECHMES, $CDFILIAL, $NRORG) {
        $primeiroDiaDTPROCESSA .= substr($DTPROCESSA, 2);
        $ultimoDiaDTPROCESSA   .= substr($DTPROCESSA, 2);

        $DTFECMES = $this->fecharDiaFactory->getDtfecmes(array("CDFILIAL"  => $CDFILIAL,
                                                               "DTINICIAL" => $primeiroDiaDTPROCESSA,
                                                               "DTFINAL"   => $ultimoDiaDTPROCESSA,
                                                               "NRORG"     => $NRORG));
                                                               
        $ultimoDiaDTPROCESSA = substr($ultimoDiaDTPROCESSA, 0, 2);

        if($DTFECMES) {
            $dia = $DTFECMES; //DateUtil::getDia(DateUtil::getDataDeString($DTFECMES));

            $diaFecMes = ($dia == DateUtil::getDia(DateUtil::getDataDeString($DTPROCESSA)));

            if(!$diaFecMes) {
                $ferifili = $this->fecharDiaFactory->getDtferifili(array("CDFILIAL"  => $CDFILIAL,
                                                                         "DTINICIAL" => $DTPROCESSA,
                                                                         "DTFINAL"   => $proxDTPROCESSA));
                
                foreach ($ferifili as $rowFerifili) {
                    if($DTFECMES == $rowFerifili['DTFERIFILI']) {
                        $diaFecMes = true;
                    }
                }
            }
        } else if(is_null($NRDIAFECHMES)) {
            $diaFecMes = (DateUtil::getMes(DateUtil::getDataDeString($DTPROCESSA)) != DateUtil::getMes(DateUtil::getDataDeString($proxDTPROCESSA)));
            
        } else {
            $dia = (($ultimoDiaDTPROCESSA != '31')
                   && (intval($ultimoDiaDTPROCESSA) < intval($NRDIAFECHMES))) ? $ultimoDiaDTPROCESSA : $NRDIAFECHMES;
            

            $diaFecMes = ($dia == DateUtil::getDia(DateUtil::getDataDeString($DTPROCESSA)));

            if(!$diaFecMes) {
                $diaDate = $dia . substr($DTPROCESSA, 2);

                $ferifili = $this->fecharDiaFactory->getDtferifili(array("CDFILIAL"  => $CDFILIAL,
                                                                         "DTINICIAL" => $DTPROCESSA,
                                                                         "DTFINAL"   => $proxDTPROCESSA));

                foreach($ferifili as $rowFerifili) {
                    if($diaDate == $rowFerifili['DTFERIFILI']) {
                        $diaFecMes = true;
                    }
                }
            }
        }
        
        return $diaFecMes;
    }

    public function exportaSAP($CDFILIAL, $DTPROCESSA, $IDEXPINFSAP) {
        if($IDEXPINFSAP == 'S') {
            $this->fecharDiaFactory->callProcedureExportaNFEntradaSAP($DTPROCESSA, $DTPROCESSA, '_'.$CDFILIAL.'_', "");

            $this->fecharDiaFactory->callProcedureExportaNFSaidaSAP($DTPROCESSA, $DTPROCESSA, '_'.$CDFILIAL.'_', "");
        }
    }

    /**
     *
     */
    public function processGr($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA, $NRDIAFECHMES, $IDGRAVATITABELA, $CDOPERADOR, $NRORG) {
        $paramsForParamsGr = array("CDFILIAL" => $CDFILIAL,
                                   "NRORG"    => $NRORG);

        $paramsGR = $this->fecharDiaFactory->getParamsGr($paramsForParamsGr);

        $tmDI  = $tiDI  =
            $tmFM  = $tiFM  =
            $tmDia = $tiDia = false;


        if(strtoupper($paramsGR['CDRELATEXP']) == 'ADM01317') {
            $tmDI  = $tiDI  = ($paramsGR['NRDIAARQEXP'] == 'DI');
            $tmFM  = $tiFM  = ($paramsGR['NRDIAARQEXP'] == 'FM');
            $tmDia = $tiDia = $paramsGR['NRDIAARQEXP'];
        } else if(strtoupper($paramsGR['CDRELATEXP']) == 'ADM01325') {
            $tiDI  = ($paramsGR['NRDIAARQEXP'] == 'DI');
            $tiFM  = ($paramsGR['NRDIAARQEXP'] == 'FM');
            $tiDia = $paramsGR['NRDIAARQEXP'];
        }

        $geraTabela = ($IDGRAVATITABELA === 'S');

        $diaMes = $this->fecharDiaFactory->getDtfecmes(array("CDFILIAL"  => $CDFILIAL,
                                                             "DTFINAL"   => $DTPROCESSA,
                                                             "DTINICIAL" => $DTPROCESSA,
                                                             "NRORG"     => $NRORG));

        $dateTimeProcessa = DateUtil::getDataDeString($DTPROCESSA);

        $primeiroDiaDTPROCESSA = DateUtil::getDia(DateUtil::getPrimeiroDiaDoMes($dateTimeProcessa));
        $ultimoDiaDTPROCESSA   = DateUtil::getDia(DateUtil::getUltimoDiaDoMes($dateTimeProcessa));
        


        if($ultimoDiaDTPROCESSA !== '31') {
            if(intval($ultimoDiaDTPROCESSA) < intval($diaMes)) {
                $dia = $ultimoDiaDTPROCESSA;
            } else {
                $dia = $diaMes;
            }
        } else {
            $dia = $diaMes;
        }

        $fechaMes = $this->fechaMesTi($DTPROCESSA, $primeiroDiaDTPROCESSA, $ultimoDiaDTPROCESSA, $proxDTPROCESSA, $diaMes, $NRDIAFECHMES, $CDFILIAL, $NRORG);

        $nrDia = DateUtil::getDia($dateTimeProcessa);

        if(($tiDI || ($tiFM && $fechaMes) || ($tiDia === $nrDia)) && $geraTabela) {
            $this->fecharDiaFactory->callProcedureExportaTabelaTi($CDFILIAL, $DTPROCESSA);

            $this->fecharDiaFactory->callProcedureExportaTiSap($DTPROCESSA, $DTPROCESSA, '_'.$CDFILIAL.'_', "");
        }

        if(($tmDI || ($tmFM && $fechaMes) || ($tmDia === $nrDia)) && $geraTabela) {
            $this->fecharDiaFactory->callProcedureExportaTransfEstoqueTm($CDFILIAL, $CDOPERADOR, $DTPROCESSA, $DTPROCESSA);

            $this->fecharDiaFactory->callProcedureExportaTmSap($CDFILIAL, $DTPROCESSA, $DTPROCESSA, "N");
        }
    }

    public function processCompass($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA, $NRDIAFECHMES, $IDGRAVATITABELA, $CDOPERADOR, $NRORG) {

        $diaMes = $this->fecharDiaFactory->getDtfecmes(array("CDFILIAL"  => $CDFILIAL,
                                                             "DTFINAL"   => $DTPROCESSA,
                                                             "DTINICIAL" => $DTPROCESSA,
                                                             "NRORG"     => $NRORG));

        $dateTimeProcessa = DateUtil::getDataDeString($DTPROCESSA);

        $primeiroDiaDTPROCESSA = DateUtil::getDia(DateUtil::getPrimeiroDiaDoMes($dateTimeProcessa));
        $ultimoDiaDTPROCESSA   = DateUtil::getDia(DateUtil::getUltimoDiaDoMes($dateTimeProcessa));
        


        if($ultimoDiaDTPROCESSA !== '31') {
            if(intval($ultimoDiaDTPROCESSA) < intval($diaMes)) {
                $dia = $ultimoDiaDTPROCESSA;
            } else {
                $dia = $diaMes;
            }
        } else {
            $dia = $diaMes;
        }

        $nrDia = DateUtil::getDia($dateTimeProcessa);

        if($nrDia) {
            
            $this->fecharDiaFactory->callProcedureCchExportaProduto();
            
            $DTPROCESSAMENOSUM = DateUtil::subtraiIntervalo($dateTimeProcessa,1);
            
            $stringData = $DTPROCESSAMENOSUM->format('d-m-Y');
            
            //$DTPROCESSAMENOSUM 3 linhas a cima chama uma função que permite subtrarir uma data com números, em seguida passo para a procedure uma string formatada no padrão BR 
            
            $this->fecharDiaFactory->callProcedureCchNfEntrada($stringData, $DTPROCESSA, '_'.$CDFILIAL.'_', "");
            
            //removido por pedido do Julian 13/08/2024
            //$this->fecharDiaFactory->callProcedureCchNfSaida($stringData, $DTPROCESSA, '_'.$CDFILIAL.'_', "");
            
        }
        if($nrDia == $ultimoDiaDTPROCESSA) {
            $this->fecharDiaFactory->callProcedureCchExportaTi($DTPROCESSA, $DTPROCESSA, '_'.$CDFILIAL.'_', "");
        }
    }

    /**
     * Essa função tem como finalidade verificar se existem notas ficasis de saída
     * que foram lançadas mas não foram emitidas. É executado um select para trazer
     * as notas nessas circustancias. Caso o select não retorne vazio, significa que
     * existem notas nessas circustancias, então é levantado um erro.
     *
     * @param $CDFILIAL | código da filial que está sendo fechado o dia
     * @param $DTPROCESSA | data de processamento daquela filial
     * @param $NRORG | NRORG
     */
    public function nfSaidaNaoEmitidas($CDFILIAL, $DTPROCESSA, $NRORG) {
        $params = array("DTENTRSAID" => $DTPROCESSA,
                        "CDFILIAL"   => $CDFILIAL,
                        "NRORG"      => $NRORG);

        $notas = $this->fecharDiaFactory->getNfsEmitidas($params);

        if(!empty($notas)) {
            throw new \Exception("Existem documentos de saída não emitidos.", 1);
        }
    }

    /**
     * Essa função tem como objetico verificar se foram gerados RE de saida e entrada
     * para a filial que se está fechando o dia. Caso um dos relátorios não tenha sido
     * gerado, é levantado um erro.
     *
     * @param $CDFILIAL | código da filial que está sendo fechado o dia
     * @param $DTPROCESSA | data de processamento daquela filial
     * @param $NRORG | NRORG
     */
    public function verificaReSaidaEntr($CDFILIAL, $DTPROCESSA, $NRORG) {
        $params = array("CDFILIAL" => $CDFILIAL,
                        "NRORG"    => $NRORG);
        
        $RE = $this->fecharDiaFactory->getReSaidaEntr($params);
        if($RE['IDCONSREFECH'] == 'S') {
            $paramsToGetRE = array("CDFILIAL"   => $CDFILIAL,
                                   "DTENTRSAID" => $DTPROCESSA,
                                   "NRORG"      => $NRORG);

            $entrada = $this->fecharDiaFactory->getReEntr($paramsToGetRE);
            $saida   = $this->fecharDiaFactory->getReSaid($paramsToGetRE);

            $isDEntradaSet = $this->array_some($entrada, function($element) {
                return $element['IDSTATUS'] == "P";
            }); 
            $isDSaidaSet = $this->array_some($saida, function($element) {
                return $element['IDSTATUS'] == "P";
            });
            if(!empty($entrada) && !empty($saida)) {
                if($isDEntradaSet == true || $isDSaidaSet == true)
                    throw new \Exception("Favor verificar se existem notas fiscais em digitação.", 1);
                else
                    throw new \Exception("RE de notas fiscais de entrada e saída devem ser emitidos.", 1);
            } else if(!empty($entrada)) {
                if($isDEntradaSet == true)
                    throw new \Exception("Favor verificar se existem notas fiscais de entrada em digitação.", 1); 
                else 
                    throw new \Exception("RE de notas fiscais de entrada deve ser emitido.", 1);   
            } else if(!empty($saida)) {
                if($isDSaidaSet == true)
                    throw new \Exception("Favor verificar se existem notas fiscais de saída em digitação.", 1);
                else 
                    throw new \Exception("RE de notas fiscais de saída deve ser emitido.", 1);
            }
        }
    }
    /**
     * Essa função tem como finalidade retornar o código do primeiro almoxarifado e
     * do primeiro local de estoque que apresentarem pendências de inventário, caso
     * aplicável, verificando corretamente a tabela com ou sem lote de acordo com a
     * parametrização recebida.
     *
     * @param $CDFILIAL | código da filial que está sendo fechado o dia
     * @param $DTPROCESSA | data de processamento daquela filial
     * @param $IDUTILLOTE | parâmetro geral que indica a utilização de lote
     */
    public function requestInventErrorCodes($CDFILIAL, $DTPROCESSA, $IDUTILLOTE) {
        $params = array("CDFILIAL"   => $CDFILIAL,
                        "DTINVEROTA" => $DTPROCESSA);

        if($IDUTILLOTE == 'S')
            return $this->fecharDiaFactory->getInventErrorCodesWithAllotment($params);
        else
            return $this->fecharDiaFactory->getInventErrorCodesWithoutAllotment($params);
    }

    public function divergenciaEstoqueCego($CDFILIAL, $DTPROCESSA) {
        $boDivergencia = false;

        $params = array("FILIPROC" => $CDFILIAL,
                        "DTPROC" => $DTPROCESSA);

        $estcegoParams = $this->fecharDiaFactory->getInventestcegoDivergenceParams($params);

        foreach ($estcegoParams as $estcegoParam) {
            $params = array("CDFILIAL"   => $CDFILIAL,
                            "DTCONTAGEM" => $estcegoParam["DTCONTAGEM"]);

            $estcegoInventory = $this->fecharDiaFactory->getInventestcegoDivergences($params);
            foreach ($estcegoInventory as $estcego) {
                if($estcego["QTINVENTESTCEGO"] != $estcego["QTESTOQDIA"])
                    return 'S';
            }
        }

        return 'N';
    }

    public function vdEstCego($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA) {
        $stDivergencia = 'N';

        $PXDTPROCESSA = DateUtil::subtraiIntervalo(DateUtil::getDataDeString($proxDTPROCESSA), 1, DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO);

        $params = array("CDFILIAL" => $CDFILIAL,
                        "DTPROCESSA" => $DTPROCESSA,
                        "PXDTPROCESSA" => $PXDTPROCESSA);

        $inventarios = $this->fecharDiaFactory->getInventestcego($params);

        foreach ($inventarios as $rowInventario) {
            $params = array("CDFILIAL" => $CDFILIAL,
                            "DTCONTAGEM" => substr($rowInventario['DTCONTAGEM'],0,10));

            $inventarioPosestdia = $this->fecharDiaFactory->getInventestcegoPosestdia($params);

            foreach ($inventarioPosestdia as $rowPosestdia) {
                $params = array("CDFILIAL" => $CDFILIAL,
                                "CDPRODUTO" => $rowPosestdia["CDPRODUTO"]);

                $parperdivestceg = $this->fecharDiaFactory->getParperdivestceg($params);

                foreach ($parperdivestceg as $rowParperdivestceg) {
                    if(abs(((($rowPosestdia["QTINVENTESTCEGO"]*100)/$rowPosestdia["QTESTOQDIA"])-100)) >= $rowParperdivestceg["VRPERDIVESTCEG"]) {
                        $params = array("CDFILIAL"        => $CDFILIAL,
                                        "DTCONTAGEM"      => $rowInventario['DTCONTAGEM'],
                                        "CDALMOXARIFE"    => $rowPosestdia["CDALMOXARIFE"],
                                        "CDLOCALESTOQ"    => $rowPosestdia["CDLOCALESTOQ"],
                                        "NRLOTEESTQ"      => $rowPosestdia["NRLOTEESTQ"],
                                        "QTPOSESTINVCEGO" => $rowPosestdia["QTESTOQDIA"],
                                        "NRSUBLOTE"       => $rowPosestdia["NRSUBLOTE"],
                                        "CDPRODUTO"       => $rowPosestdia["CDPRODUTO"]);

                        $this->fecharDiaFactory->updateIteminventcego($params);

                        if($rowPosestdia["QTINVENTESTCEGO"] != $rowPosestdia["QTESTOQDIA"])
                            $stDivergencia = 'S';
                    }
                }
            }
        }

        $result = array("inventarios"    => $inventarios,
                        "stDivergencia" => $stDivergencia);
        return $result;
    }

    /**
     * Essa função tem como finalidade retornar todos os parâmetros gerais e de
     * unidade necessários para a validação dos procedimentos envolvidos no
     * fechamento do dia.
     *
     * @param $CDFILIAL | código da unidade em que está sendo fechado o dia
     */
    public function requestParameters($CDFILIAL) {
        $params = $this->fecharDiaFactory->getParameters($CDFILIAL);

        return $params;
    }

    /**
     * Essa função tem como finalidade retornar todos os parâmetros gerais e de
     * unidade necessários para a validação dos procedimentos envolvidos no
     * fechamento do dia.
     *
     * @param $IDCONSISTECONSO | parâmetro que indica se aquela unidade consiste
     *                           consolidação de vendas
     * @param $CDFILIAL        | código da unidade em que está sendo fechado o dia
     * @param $DTPROCESSA      | data de processamento daquela unidade
     */
    public function verificaConsolidacaoDeVendas($IDCONSISTECONSO, $CDFILIAL, $DTPROCESSA) {
        if($IDCONSISTECONSO == 'S') {
            $params = array("CDFILIAL"   => $CDFILIAL,
                            "DTPROCESSA" => $DTPROCESSA);

            $vendasNaoConsolidadas = $this->fecharDiaFactory->getVendasNaoConsolidadas($params);

            if(!empty($vendasNaoConsolidadas)) {
                throw new \Exception("Existem vendas para a data de processamento da filial que ainda não foram consolidadas.", 1);
            }
        }
    }

    /**
     * Essa função tem como finalidade validar o bloqueio de operações por
     * falta de emissão de notas de saída, conforme parametrização.
     *
     * @param $IDBLFECHSNFE | parâmetro que indica se aquela unidade requer o
     *                        bloqueio por notas de saída não emitidas.
     * @param $CDFILIAL     | código da unidade em que está sendo fechado o dia
     * @param $DTPROCESSA   | data de processamento daquela unidade
     */
    public function verificaNFSaida($IDBLFECHSNFE, $CDFILIAL, $DTENTRSAID) {
        if($IDBLFECHSNFE == 'S') {
            $params = array("CDFILIAL"   => $CDFILIAL,
                            "DTENTRSAID" => $DTENTRSAID);

            $saidasNaoEmitidas = $this->fecharDiaFactory->getSaidasNaoEmitidas($params);

            if(!empty($saidasNaoEmitidas)) {
                throw new \Exception("Existem documentos de saída não emitidos.", 1);
            }
        }
    }
    
    public function verificaNFEmDigitacao($IDBLFECHNFDIG, $CDFILIAL, $DTENTRSAID) {
        if($IDBLFECHNFDIG == 'S') {
            $params = array("CDFILIAL"   => $CDFILIAL,
                            "DTENTRSAID" => $DTENTRSAID);

            $nfEmDigitacao = $this->fecharDiaFactory->getNFEmDigitacao($params);
            
            if(!empty($nfEmDigitacao)) {
                throw new \Exception("Existem Documentos em Digitação.", 1);
            }
        }
    }
    
    public function verificaNFEntrada($IDBLFECHSNFE, $CDFILIAL, $DTPROCESSA) {
        if($IDBLFECHSNFE == 'S') {
            $params = array("P_CDFILIAL"   => $CDFILIAL,
                            "P_DTPROCESSA" => $DTPROCESSA);

            $entradasNaoEmitidas = $this->fecharDiaFactory->getEntradasNaoEmitidas($params);

            if(!empty($entradasNaoEmitidas)) {
                throw new \Exception("Existem documentos de saída e/ou entrada não emitidos/transmitidos.", 1);
            }
        }
    }
    
    public function verificaNFAtrasada($IDBLOQFECHNF, $CDFILIAL, $DTPROCESSA) {
        if($IDBLOQFECHNF == 'S') {
            $params = array("P_CDFILIAL"   => $CDFILIAL,
                            "P_DTPROCESSA" => $DTPROCESSA);

            $entradasNaoEmitidas = $this->fecharDiaFactory->getNFAtrasadas($params);

            if(!empty($entradasNaoEmitidas)) {
                throw new \Exception("Existe entrada sem nota fiscal que não possui nota fiscal atrasada.", 1);
            }
        }
    }
    
    public function getDadosFecharDia($CDFILIAL) {
        $dadosFilial = $this->fecharDiaFactory->getDadosFilial($CDFILIAL);
        if($dadosFilial) return $dadosFilial;
        else return array();
        
    }
    
    public function validaImplatacaoAjusteDesatualizado($CDFILIAL, $DTPROCESSA){
        $FilialNaoAtualizada = $this->fecharDiaFactory->validaImplatacaoAjusteDesatualizado($CDFILIAL, $DTPROCESSA);
        if(!empty($FilialNaoAtualizada)){
            throw new \Exception("Operação bloqueada! Existe(m) produto(s) pertencentes a implantação e/ou ajustes não evoluído(s) no estoque");
        }
    }
    
    public function CheckEffectiveDays($CDFILIAL, $DTPROCESSA){
        $procedureResults = $this->fecharDiaFactory->callProcedureValidaCardapioFecdia($CDFILIAL, $DTPROCESSA);
        foreach ($procedureResults as $resultName => $resultValue){
            switch($resultName){
                case 'P_SEMEFETIVOS':
                    if(($resultValue) == 'S') throw new \Exception("Não é permitido fazer o fechamento do dia sem que os comensais efetivos sejam lançados para todos os serviços da unidade.",1);
                    break;
                case 'P_SEMRETIRADA':
                    if(($resultValue) == 'S') throw new \Exception("Não é possível realizar o fechamento do dia sem antes efetuar o lançamento de retirada.",1);
                    break;
                case 'P_NAOEVOLESTOQ':
                    if(($resultValue) == 'S') throw new \Exception("Não é possível realizar o fechamento do dia sem antes atualizar os itens da retirada.",1);
                    break;
                case 'P_SERVSEMRETI':
                    if(($resultValue) == 'S') throw new \Exception("Não é possível realizar o fechamento do dia sem antes efetuar o lançamento de retirada para todos os serviços.",1);
                    break;
                case 'P_CDSERVDIFEFET':
                    if(!is_null($resultValue)){
                        
                        $servicesCode = explode('_',substr($resultValue,1));
                        $errorMessage = "Efetivos planejados diferem dos efetivos faturados para os seguintes serviços:<br><br>";
                        try {
                            foreach($servicesCode as $serviceCode){
                                $serviceData = $this->fecharDiaFactory->getServiceName($CDFILIAL,$serviceCode);
                                $errorMessage .= "{$serviceData['CDSERVFILI']} - {$serviceData['NMSERVICO']}<br>";
                            }
                        }catch(\Exception $e){
                            $errorMessage.= "Não foi possivel localizar nome do serviço com os parâmetros: ".$resultValue;
                        }
                        throw new \Exception($errorMessage);
                    }
                    break;
                default:
                    break;
            }
        }
    }
    public function array_some(array $paramsToGetRE, callable $fn) {
        if (empty($paramsToGetRE)) {
            return "O array está vazio.";
        }
        foreach ($paramsToGetRE as $value) {
            if ($fn($value)) {
                return true;
            }
        }
        return false;
    }
}